@@ -7,6 +7,7 @@ import pathspec
 
 from ..utils.fs import locate_file
 from .constants import DEFAULT_BUILD_DIRECTORY, BuildEnvVars
+from .utils import normalize_relative_directory, normalize_relative_path
 
 
 class BuilderConfig(object):
@@ -19,8 +20,8 @@ class BuilderConfig(object):
         self.__hook_config = None
         self.__versions = None
         self.__dependencies = None
+        self.__sources = None
         self.__packages = None
-        self.__package_sources = None
 
         # Possible pathspec.PathSpec
         self.__include_spec = None
@@ -39,6 +40,8 @@ class BuilderConfig(object):
         self.__only_packages = None
         self.__reproducible = None
         self.__dev_mode_dirs = None
+        self.__dev_mode_exact = None
+        self.__require_runtime_dependencies = None
 
     @property
     def builder(self):
@@ -281,6 +284,25 @@ class BuilderConfig(object):
 
         return self.__ignore_vcs
 
+    @property
+    def require_runtime_dependencies(self):
+        if self.__require_runtime_dependencies is None:
+            if 'require-runtime-dependencies' in self.target_config:
+                require_runtime_dependencies = self.target_config['require-runtime-dependencies']
+                if not isinstance(require_runtime_dependencies, bool):
+                    raise TypeError(
+                        'Field `tool.hatch.build.targets.{}.require-runtime-dependencies` '
+                        'must be a boolean'.format(self.plugin_name)
+                    )
+            else:
+                require_runtime_dependencies = self.build_config.get('require-runtime-dependencies', False)
+                if not isinstance(require_runtime_dependencies, bool):
+                    raise TypeError('Field `tool.hatch.build.require-runtime-dependencies` must be a boolean')
+
+            self.__require_runtime_dependencies = require_runtime_dependencies
+
+        return self.__require_runtime_dependencies
+
     @property
     def only_packages(self):
         """
@@ -356,6 +378,24 @@ class BuilderConfig(object):
 
         return self.__dev_mode_dirs
 
+    @property
+    def dev_mode_exact(self):
+        if self.__dev_mode_exact is None:
+            if 'dev-mode-exact' in self.target_config:
+                dev_mode_exact = self.target_config['dev-mode-exact']
+                if not isinstance(dev_mode_exact, bool):
+                    raise TypeError(
+                        'Field `tool.hatch.build.targets.{}.dev-mode-exact` must be a boolean'.format(self.plugin_name)
+                    )
+            else:
+                dev_mode_exact = self.build_config.get('dev-mode-exact', False)
+                if not isinstance(dev_mode_exact, bool):
+                    raise TypeError('Field `tool.hatch.build.dev-mode-exact` must be a boolean')
+
+            self.__dev_mode_exact = dev_mode_exact
+
+        return self.__dev_mode_exact
+
     @property
     def versions(self):
         if self.__versions is None:
@@ -435,7 +475,16 @@ class BuilderConfig(object):
 
                 dependencies[dependency] = None
 
+            require_runtime_dependencies = self.require_runtime_dependencies
             for hook_name, config in self.hook_config.items():
+                hook_require_runtime_dependencies = config.get('require-runtime-dependencies', False)
+                if not isinstance(hook_require_runtime_dependencies, bool):
+                    raise TypeError(
+                        'Option `require-runtime-dependencies` of build hook `{}` must be a boolean'.format(hook_name)
+                    )
+                elif hook_require_runtime_dependencies:
+                    require_runtime_dependencies = True
+
                 hook_dependencies = config.get('dependencies', [])
                 if not isinstance(hook_dependencies, list):
                     raise TypeError('Option `dependencies` of build hook `{}` must be an array'.format(hook_name))
@@ -450,10 +499,65 @@ class BuilderConfig(object):
 
                     dependencies[dependency] = None
 
+            if require_runtime_dependencies:
+                for dependency in self.builder.metadata.core.dependencies:
+                    dependencies[dependency] = None
+
             self.__dependencies = list(dependencies)
 
         return self.__dependencies
 
+    @property
+    def sources(self):
+        if self.__sources is None:
+            if 'sources' in self.target_config:
+                sources_config = self.target_config
+                sources_location = 'tool.hatch.build.targets.{}.sources'.format(self.plugin_name)
+            else:
+                sources_config = self.build_config
+                sources_location = 'tool.hatch.build.sources'
+
+            sources = {}
+
+            raw_sources = sources_config.get('sources', [])
+            if isinstance(raw_sources, list):
+                for i, source in enumerate(raw_sources, 1):
+                    if not isinstance(source, str):
+                        raise TypeError('Source #{} in field `{}` must be a string'.format(i, sources_location))
+                    elif not source:
+                        raise ValueError(
+                            'Source #{} in field `{}` cannot be an empty string'.format(i, sources_location)
+                        )
+
+                    sources[normalize_relative_directory(source)] = ''
+            elif isinstance(raw_sources, dict):
+                for i, (source, path) in enumerate(raw_sources.items(), 1):
+                    if not source:
+                        raise ValueError(
+                            'Source #{} in field `{}` cannot be an empty string'.format(i, sources_location)
+                        )
+                    elif not isinstance(path, str):
+                        raise TypeError(
+                            'Path for source `{}` in field `{}` must be a string'.format(source, sources_location)
+                        )
+
+                    normalized_path = normalize_relative_path(path)
+                    if normalized_path:
+                        normalized_path += os.path.sep
+
+                    sources[normalize_relative_directory(source)] = normalized_path
+            else:
+                raise TypeError('Field `{}` must be a mapping or array of strings'.format(sources_location))
+
+            for relative_path in self.packages:
+                source, package = os.path.split(relative_path)
+                if source and normalize_relative_directory(relative_path) not in sources:
+                    sources[normalize_relative_directory(source)] = ''
+
+            self.__sources = dict(sorted(sources.items()))
+
+        return self.__sources
+
     @property
     def packages(self):
         if self.__packages is None:
@@ -464,8 +568,6 @@ class BuilderConfig(object):
                 package_config = self.build_config
                 package_location = 'tool.hatch.build.packages'
 
-            all_packages = set()
-
             packages = package_config.get('packages', self.default_packages())
             if not isinstance(packages, list):
                 raise TypeError('Field `{}` must be an array of strings'.format(package_location))
@@ -476,45 +578,15 @@ class BuilderConfig(object):
                 elif not package:
                     raise ValueError('Package #{} in field `{}` cannot be an empty string'.format(i, package_location))
 
-                all_packages.add(os.path.normpath(package).lstrip(os.path.sep))
-
-            unique_packages = {}
-            packages = []
-            package_sources = []
-
-            for relative_path in sorted(all_packages):
-                source, package = os.path.split(relative_path)
-                if package in unique_packages:
-                    raise ValueError(
-                        'Package `{}` of field `{}` is already defined by path `{}`'.format(
-                            package, package_location, unique_packages[package]
-                        )
-                    )
-
-                unique_packages[package] = relative_path
-
-                if source:
-                    package_sources.append(source + os.path.sep)
-
-                packages.append(relative_path)
-
-            self.__packages = packages
-            self.__package_sources = package_sources
+            self.__packages = sorted(normalize_relative_path(package) for package in packages)
 
         return self.__packages
 
-    @property
-    def package_sources(self):
-        if self.__package_sources is None:
-            _ = self.packages
-
-        return self.__package_sources
-
     def get_distribution_path(self, relative_path):
         # src/foo/bar.py -> foo/bar.py
-        for package_source in self.package_sources:
-            if relative_path.startswith(package_source):
-                return relative_path[len(package_source) :]
+        for source, replacement in self.sources.items():
+            if relative_path.startswith(source):
+                return relative_path.replace(source, replacement, 1)
 
         return relative_path
 
