@@ -1,10 +1,10 @@
 import os
-import re
 import sys
 from collections import OrderedDict
 from copy import deepcopy
 
 from ..utils.fs import locate_file
+from .utils import get_normalized_dependency, is_valid_project_name, normalize_project_name
 
 # TODO: remove when we drop Python 2
 if sys.version_info[0] < 3:  # no cov
@@ -54,7 +54,7 @@ class ProjectMetadata(object):
     @property
     def version(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#version
+        https://peps.python.org/pep-0621/#version
         """
         if self._version is None:
             self._set_version()
@@ -94,9 +94,9 @@ class ProjectMetadata(object):
             if not isinstance(core_metadata, dict):
                 raise TypeError('The `project` configuration must be a table')
 
-            metadata = CoreMetadata(self.root, core_metadata)
+            metadata = CoreMetadata(self.root, core_metadata, self.hatch.metadata)
 
-            metadata_hooks = self.hatch.metadata_hooks
+            metadata_hooks = self.hatch.metadata.hooks
             if metadata_hooks:
                 static_fields = set(core_metadata)
                 self._set_version(metadata)
@@ -159,7 +159,7 @@ class ProjectMetadata(object):
 
 class BuildMetadata(object):
     """
-    https://www.python.org/dev/peps/pep-0517/
+    https://peps.python.org/pep-0517/
     """
 
     def __init__(self, root, config):
@@ -231,12 +231,13 @@ class BuildMetadata(object):
 
 class CoreMetadata(object):
     """
-    https://www.python.org/dev/peps/pep-0621/
+    https://peps.python.org/pep-0621/
     """
 
-    def __init__(self, root, config):
+    def __init__(self, root, config, hatch_metadata):
         self.root = root
         self.config = config
+        self.hatch_metadata = hatch_metadata
 
         self._name = None
         self._version = None
@@ -267,7 +268,7 @@ class CoreMetadata(object):
     @property
     def name(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#name
+        https://peps.python.org/pep-0621/#name
         """
         if self._name is None:
             if 'name' in self.dynamic:
@@ -282,22 +283,20 @@ class CoreMetadata(object):
             elif not isinstance(name, str):
                 raise TypeError('Field `project.name` must be a string')
 
-            # https://www.python.org/dev/peps/pep-0508/#names
-            if not re.search('^([A-Z0-9]|[A-Z0-9][A-Z0-9._-]*[A-Z0-9])$', name, re.IGNORECASE):
+            if not is_valid_project_name(name):
                 raise ValueError(
-                    'Required field `project.name` must only contain ASCII letters/digits, '
-                    'underscores, hyphens, and periods.'
+                    'Required field `project.name` must only contain ASCII letters/digits, underscores, '
+                    'hyphens, and periods, and must begin and end with ASCII letters/digits.'
                 )
 
-            # https://www.python.org/dev/peps/pep-0503/#normalized-names
-            self._name = re.sub(r'[-_.]+', '-', name).lower()
+            self._name = normalize_project_name(name)
 
         return self._name
 
     @property
     def version(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#version
+        https://peps.python.org/pep-0621/#version
         """
         if self._version is None:
             if 'version' not in self.config:
@@ -324,7 +323,7 @@ class CoreMetadata(object):
     @property
     def description(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#description
+        https://peps.python.org/pep-0621/#description
         """
         if self._description is None:
             if 'description' in self.config:
@@ -347,7 +346,7 @@ class CoreMetadata(object):
     @property
     def readme(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#readme
+        https://peps.python.org/pep-0621/#readme
         """
         if self._readme is None:
             if 'readme' in self.config:
@@ -436,7 +435,7 @@ class CoreMetadata(object):
     @property
     def readme_content_type(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#readme
+        https://peps.python.org/pep-0621/#readme
         """
         if self._readme_content_type is None:
             _ = self.readme
@@ -446,7 +445,7 @@ class CoreMetadata(object):
     @property
     def readme_path(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#readme
+        https://peps.python.org/pep-0621/#readme
         """
         if self._readme_path is None:
             _ = self.readme
@@ -456,7 +455,7 @@ class CoreMetadata(object):
     @property
     def requires_python(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#requires-python
+        https://peps.python.org/pep-0621/#requires-python
         """
         if self._requires_python is None:
             from packaging.specifiers import InvalidSpecifier, SpecifierSet
@@ -493,7 +492,7 @@ class CoreMetadata(object):
     @property
     def license(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#license
+        https://peps.python.org/pep-0621/#license
         """
         if self._license is None:
             if 'license' in self.config:
@@ -546,7 +545,7 @@ class CoreMetadata(object):
     @property
     def license_expression(self):
         """
-        https://www.python.org/dev/peps/pep-0639/
+        https://peps.python.org/pep-0639/
         """
         if self._license_expression is None:
             _ = self.license
@@ -556,7 +555,7 @@ class CoreMetadata(object):
     @property
     def license_files(self):
         """
-        https://www.python.org/dev/peps/pep-0639/
+        https://peps.python.org/pep-0639/
         """
         if self._license_files is None:
             if 'license-files' not in self.config:
@@ -620,7 +619,7 @@ class CoreMetadata(object):
     @property
     def authors(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#authors-maintainers
+        https://peps.python.org/pep-0621/#authors-maintainers
         """
         if self._authors is None:
             if 'authors' in self.config:
@@ -680,7 +679,7 @@ class CoreMetadata(object):
     @property
     def authors_data(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#authors-maintainers
+        https://peps.python.org/pep-0621/#authors-maintainers
         """
         if self._authors_data is None:
             _ = self.authors
@@ -690,7 +689,7 @@ class CoreMetadata(object):
     @property
     def maintainers(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#authors-maintainers
+        https://peps.python.org/pep-0621/#authors-maintainers
         """
         if self._maintainers is None:
             if 'maintainers' in self.config:
@@ -750,7 +749,7 @@ class CoreMetadata(object):
     @property
     def maintainers_data(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#authors-maintainers
+        https://peps.python.org/pep-0621/#authors-maintainers
         """
         if self._maintainers_data is None:
             _ = self.maintainers
@@ -760,7 +759,7 @@ class CoreMetadata(object):
     @property
     def keywords(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#keywords
+        https://peps.python.org/pep-0621/#keywords
         """
         if self._keywords is None:
             if 'keywords' in self.config:
@@ -791,7 +790,7 @@ class CoreMetadata(object):
     @property
     def classifiers(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#classifiers
+        https://peps.python.org/pep-0621/#classifiers
         """
         if self._classifiers is None:
             if 'classifiers' in self.config:
@@ -822,7 +821,7 @@ class CoreMetadata(object):
     @property
     def urls(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#urls
+        https://peps.python.org/pep-0621/#urls
         """
         if self._urls is None:
             if 'urls' in self.config:
@@ -852,7 +851,7 @@ class CoreMetadata(object):
     @property
     def scripts(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#entry-points
+        https://peps.python.org/pep-0621/#entry-points
         """
         if self._scripts is None:
             if 'scripts' in self.config:
@@ -883,7 +882,7 @@ class CoreMetadata(object):
     @property
     def gui_scripts(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#entry-points
+        https://peps.python.org/pep-0621/#entry-points
         """
         if self._gui_scripts is None:
             if 'gui-scripts' in self.config:
@@ -916,7 +915,7 @@ class CoreMetadata(object):
     @property
     def entry_points(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#entry-points
+        https://peps.python.org/pep-0621/#entry-points
         """
         if self._entry_points is None:
             if 'entry-points' in self.config:
@@ -966,7 +965,7 @@ class CoreMetadata(object):
     @property
     def dependencies_complex(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#dependencies-optional-dependencies
+        https://peps.python.org/pep-0621/#dependencies-optional-dependencies
         """
         if self._dependencies_complex is None:
             from packaging.requirements import InvalidRequirement, Requirement
@@ -984,23 +983,23 @@ class CoreMetadata(object):
             if not isinstance(dependencies, list):
                 raise TypeError('Field `project.dependencies` must be an array')
 
-            dependencies_complex = []
+            dependencies_complex = OrderedDict()
 
             for i, entry in enumerate(dependencies, 1):
                 if not isinstance(entry, str):
                     raise TypeError('Dependency #{} of field `project.dependencies` must be a string'.format(i))
 
                 try:
-                    dependency = Requirement(entry)
+                    requirement = Requirement(entry)
                 except InvalidRequirement as e:
                     raise ValueError('Dependency #{} of field `project.dependencies` is invalid: {}'.format(i, e))
                 else:
-                    if dependency.url:
+                    if requirement.url and not self.hatch_metadata.allow_direct_references:
                         raise ValueError(
                             'Dependency #{} of field `project.dependencies` cannot be a direct reference'.format(i)
                         )
 
-                    dependencies_complex.append(dependency)
+                    dependencies_complex[get_normalized_dependency(requirement)] = requirement
 
             self._dependencies_complex = dependencies_complex
 
@@ -1009,17 +1008,17 @@ class CoreMetadata(object):
     @property
     def dependencies(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#dependencies-optional-dependencies
+        https://peps.python.org/pep-0621/#dependencies-optional-dependencies
         """
         if self._dependencies is None:
-            self._dependencies = sorted(map(str, self.dependencies_complex), key=lambda d: d.lower())
+            self._dependencies = sorted(self.dependencies_complex)
 
         return self._dependencies
 
     @property
     def optional_dependencies(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#dependencies-optional-dependencies
+        https://peps.python.org/pep-0621/#dependencies-optional-dependencies
         """
         if self._optional_dependencies is None:
             from packaging.requirements import InvalidRequirement, Requirement
@@ -1037,16 +1036,23 @@ class CoreMetadata(object):
             if not isinstance(optional_dependencies, dict):
                 raise TypeError('Field `project.optional-dependencies` must be a table')
 
-            optional_dependency_entries = OrderedDict()
+            normalized_options = {}
+            optional_dependency_entries = {}
 
-            for option, dependencies in sorted(optional_dependencies.items()):
-                if not isinstance(dependencies, list):
+            for option, dependencies in optional_dependencies.items():
+                if not is_valid_project_name(option):
+                    raise ValueError(
+                        'Optional dependency group `{}` of field `project.optional-dependencies` must only contain '
+                        'ASCII letters/digits, underscores, hyphens, and periods, and must begin and end with '
+                        'ASCII letters/digits.'.format(option)
+                    )
+                elif not isinstance(dependencies, list):
                     raise TypeError(
                         'Dependencies for option `{}` of field `project.optional-dependencies` '
                         'must be an array'.format(option)
                     )
 
-                entries = []
+                entries = set()
 
                 for i, entry in enumerate(dependencies, 1):
                     if not isinstance(entry, str):
@@ -1056,31 +1062,39 @@ class CoreMetadata(object):
                         )
 
                     try:
-                        dependency = Requirement(entry)
+                        requirement = Requirement(entry)
                     except InvalidRequirement as e:
                         raise ValueError(
                             'Dependency #{} of option `{}` of field `project.optional-dependencies` '
                             'is invalid: {}'.format(i, option, e)
                         )
                     else:
-                        if dependency.url:
+                        if requirement.url and not self.hatch_metadata.allow_direct_references:
                             raise ValueError(
                                 'Dependency #{} of option `{}` of field `project.optional-dependencies` '
                                 'cannot be a direct reference'.format(i, option)
                             )
 
-                        entries.append(entry)
+                        entries.add(get_normalized_dependency(requirement))
 
-                optional_dependency_entries[option] = sorted(entries, key=lambda s: s.lower())
+                normalized_option = normalize_project_name(option)
+                if normalized_option in normalized_options:
+                    raise ValueError(
+                        'Optional dependency groups `{}` and `{}` of field `project.optional-dependencies` both '
+                        'evaluate to `{}`.'.format(normalized_options[normalized_option], option, normalized_option)
+                    )
 
-            self._optional_dependencies = optional_dependency_entries
+                normalized_options[normalized_option] = option
+                optional_dependency_entries[normalized_option] = sorted(entries)
+
+            self._optional_dependencies = OrderedDict(sorted(optional_dependency_entries.items()))
 
         return self._optional_dependencies
 
     @property
     def dynamic(self):
         """
-        https://www.python.org/dev/peps/pep-0621/#dynamic
+        https://peps.python.org/pep-0621/#dynamic
         """
         if self._dynamic is None:
             dynamic = self.config.get('dynamic', [])
@@ -1107,40 +1121,22 @@ class HatchMetadata(object):
         self.config = config
         self.plugin_manager = plugin_manager
 
-        self._metadata_config = None
-        self._metadata_hooks = None
+        self._metadata = None
         self._build_config = None
         self._build_targets = None
         self._version_source = None
         self._version = None
 
     @property
-    def metadata_config(self):
-        if self._metadata_config is None:
+    def metadata(self):
+        if self._metadata is None:
             metadata_config = self.config.get('metadata', {})
             if not isinstance(metadata_config, dict):
                 raise TypeError('Field `tool.hatch.metadata` must be a table')
 
-            self._metadata_config = metadata_config
+            self._metadata = HatchMetadataSettings(self.root, metadata_config, self.plugin_manager)
 
-        return self._metadata_config
-
-    @property
-    def metadata_hooks(self):
-        if self._metadata_hooks is None:
-            hook_config = self.metadata_config
-
-            configured_metadata_hooks = OrderedDict()
-            for hook_name, config in hook_config.items():
-                metadata_hook = self.plugin_manager.metadata_hook.get(hook_name)
-                if metadata_hook is None:
-                    raise ValueError('Unknown metadata hook: {}'.format(hook_name))
-
-                configured_metadata_hooks[hook_name] = metadata_hook(self.root, config)
-
-            self._metadata_hooks = configured_metadata_hooks
-
-        return self._metadata_hooks
+        return self._metadata
 
     @property
     def build_config(self):
@@ -1203,3 +1199,53 @@ class HatchMetadata(object):
             self._version_source = version_source(self.root, options)
 
         return self._version_source
+
+
+class HatchMetadataSettings(object):
+    def __init__(self, root, config, plugin_manager):
+        self.root = root
+        self.config = config
+        self.plugin_manager = plugin_manager
+
+        self._allow_direct_references = None
+        self._hook_config = None
+        self._hooks = None
+
+    @property
+    def allow_direct_references(self):
+        if self._allow_direct_references is None:
+            allow_direct_references = self.config.get('allow-direct-references', False)
+            if not isinstance(allow_direct_references, bool):
+                raise TypeError('Field `tool.hatch.metadata.allow-direct-references` must be a boolean')
+
+            self._allow_direct_references = allow_direct_references
+
+        return self._allow_direct_references
+
+    @property
+    def hook_config(self):
+        if self._hook_config is None:
+            hook_config = self.config.get('hooks', {})
+            if not isinstance(hook_config, dict):
+                raise TypeError('Field `tool.hatch.metadata.hooks` must be a table')
+
+            self._hook_config = hook_config
+
+        return self._hook_config
+
+    @property
+    def hooks(self):
+        if self._hooks is None:
+            hook_config = self.hook_config
+
+            configured_hooks = OrderedDict()
+            for hook_name, config in hook_config.items():
+                metadata_hook = self.plugin_manager.metadata_hook.get(hook_name)
+                if metadata_hook is None:
+                    raise ValueError('Unknown metadata hook: {}'.format(hook_name))
+
+                configured_hooks[hook_name] = metadata_hook(self.root, config)
+
+            self._hooks = configured_hooks
+
+        return self._hooks
